<?php
/**
 * @package    Proxim
 * @author     Davison Pro <davisonprp.coder@gmail.com | https://davisonprp.dev>
 * @copyright  2019 Proxim
 * @version    1.0.0
 * @since      File available since Release 1.0.0
 */

namespace Proxim\Api;

use Db;
use Exception;
use PrePaper;
use PrePaperPresenter;
use Proxim\Configuration;
use Proxim\Currency;
use Proxim\Database\DbQuery;
use Proxim\Module\Module;
use Proxim\Route;
use Proxim\User\Customer;
use Proxim\User\Wallet;
use Proxim\Util\ArrayUtils;
use Proxim\Validate;

class Papers extends Route {
    public function getPaper( $paperId ) {
        $app = $this->app;
        $user = $app->container->user;
        
        if(!Module::getInstanceByName('prepapers')) {
            return $app->response([
                "success" => false,
				"errors" => "Invalid paper id: %d"
            ]);
        }

		$prepaper = new PrePaper( (int) $paperId);
		if (!Validate::isLoadedObject($prepaper) ) {
			return $app->response([
				"success" => false,
				"errors" => "Invalid paper id: %d"
			]);
        }

        $currency = Currency::getDefaultCurrency();

        $paper = array(
            'id' => (int) $prepaper->id,
            'title' => $prepaper->title,
            'total_charge' => (float) $prepaper->price,
            'currency_id' => (int) $currency->id,
            'currency_rate' => (float) $currency->conversion_rate
        );

        $orderId = (int) Db::getInstance()->getValue('
            SELECT order_id
            FROM ' . Db::prefix('prepaper_payment') . '
            WHERE prepaper_id = ' . (int) $prepaper->id .  '
            AND customer_id = '. (int) $user->id
        ); 

        if($orderId) {
            $paper['order_id'] = (int) $orderId;
        }
        
		return $app->response([
			"success" => true,
			"paper" => $paper
		]);
    }

    public function proceedPaperPayment( $paperId ) {
        $app = $this->app;
        $user = $app->container->user;
        $params = $app->request->post();

        if(!Module::getInstanceByName('prepapers')) {
            return $app->response([
                "success" => false,
				"errors" => "Invalid paper id: " . $paperId
            ]);
        }

        $payment_method = ArrayUtils::get($params, 'payment_method_value');

		$prepaper = new PrePaper( (int) $paperId );
		if ( !Validate::isLoadedObject($prepaper) ) {
			return $app->response([
				"success" => false,
				"errors" => "Invalid paper id: " . $paperId
			]);
        }

        if( $payment_method == "wallet" ) {
            $paperUri = Configuration::get('SITE_DOMAIN') . '/dashboard/order/' . $prepaper->order_id;
            $currency = Currency::getDefaultCurrency();

            try {
                $success =  Customer::chargeWallet( 
                    $user->id, 
                    $prepaper->price, 
                    $currency->id,
                    $prepaper->id,
                    'paper' 
                );

                if($success) {
                    $redirectUri = Configuration::get('SITE_DOMAIN') . '/payment/wallet/success?redirectUri=' . $paperUri;
                } else {
                    $redirectUri = Configuration::get('SITE_DOMAIN') . '/payment/wallet/failed?redirectUri=' . $paperUri;
                }
            } catch( Exception $e ) {
                return $app->response([
                    "success" => false,
                    "errors" => [
                        $e->getMessage()
                    ]
                ]);
            }

        } else {
            $redirectUri = Configuration::get('SITE_DOMAIN') . '/checkout/' . $payment_method . '/paper/' . $prepaper->id;
        }

		return $app->response([
			"success" => true,
			'redirect_to' => $redirectUri,
		]);
    }
}