<?php 
/**
 * @package    Proxim
 * @author     Davison Pro <davisonpro.coder@gmail.com | https://davisonpro.dev>
 * @copyright  2019 Proxim
 * @version    1.0.0
 * @since      File available since Release 1.0.0
 */

namespace Proxim\Api;

use Db;
use Proxim\Coupon;
use Proxim\Crypto\Hashing;
use Proxim\Database\DbQuery;
use Proxim\Hook;
use Proxim\Order\Order;
use Proxim\Route;
use Proxim\Site\Site;
use Proxim\Tools;
use Proxim\User\Customer;
use Proxim\Util\ArrayUtils;
use Proxim\Util\DateUtils;
use Proxim\Validate;

class Subscribe extends Route {
    public function subscribe() {
        $app = $this->app;
		$payload = $app->request->post();
        $user = $app->container->user;
        $site = $app->container->site;

        $name = ArrayUtils::get($payload, 'name', 'Customer');
        $email = ArrayUtils::get($payload, 'email');
        $type = ArrayUtils::get($payload, 'type');

        /** Email Validation */
        if ( !$email ) {
            return $app->response([
                'success' => false,
                'message' => 'Email is required'
            ]);
        }

        if(!validate::isEmail($email)) {
            return $app->response([
                'success' => false,
                'message' => 'Enter a valid Email address'
            ]);
        }

        $sql = new DbQuery();
        $sql->select('m.mail_id');
        $sql->from('mailing_list', 'm');
        $sql->where('m.`email` = \'' . pSQL($email) . '\'');
        $mailAddress = Db::getInstance(PROX_USE_SQL_SLAVE)->getValue( $sql );

        if($mailAddress) {
            return $app->response([
                'success' => false,
                'message' => 'used'
            ]);
        }

        Db::getInstance()->insert(
            'mailing_list',
            array(
                'site_id' => $site->id,
                'name' => $name,
                'email' => $email,
                'date_add' => DateUtils::now()
            )
        );

        switch($type) {
            case 'discount':
                $couponCode = Tools::randomGen();
                $couponCode = Tools::strtoupper($couponCode);
                $couponDiscount = ArrayUtils::get($payload, 'couponDiscount');
                
                $coupon_expiry = date('Y-m-d H:i:s', strtotime('+30 days'));

                $discount = new Coupon();
                $discount->site_id = $site->id;
                $discount->coupon_type =  Coupon::PERCENT;
                $discount->coupon_code = $couponCode;
                $discount->coupon_value = $couponDiscount;
                $discount->service_type_id = Order::COMPLEX_ASSIGNMENT;
                $discount->active = true;
                $discount->expiry = $coupon_expiry;
                $discount->add();

                return $app->response([
                    "success" => true,
                    "couponCode" => $discount->coupon_code
                ]);	
                break;
        }

        return $app->response([
            "success" => true
        ]);	
    }
}