<?php

/**
 * @package    Proxim
 * @author     Davison Pro <davis@davisonpro.dev | https://davisonpro.dev>
 * @copyright  2019 Proxim
 * @version    1.5.0
 * @since      File available since Release 1.0.0
 */

use Proxim\Application;
use Proxim\Mail;
use Proxim\MailTemplate;
use Proxim\Validate;
use Proxim\Configuration;
use Proxim\Module\Module;
use Proxim\User\Customer;
use Proxim\Util\ArrayUtils;

define('CURRENT_CSE_MODULE_DIR', realpath(dirname(__FILE__)));

class Customer_Signup_Email extends Module
{
    const TEMPLATE_CUSTOMER_SIGNUP = 'customer_signup';

    public function __construct()
    {
        $this->name = 'customer_signup_email';
        $this->icon = 'fa fa-envelope';
        $this->version = '1.0.0';
        $this->prox_versions_compliancy = array('min' => '1.0.0', 'max' => Application::VERSION);
        $this->author = 'Davison Pro';
        $this->displayName = 'Customer Signup Email';
        $this->description = 'Receive customer signup emails and notifications.';

        $this->bootstrap = true;
        parent::__construct();
    }

    public function hookActionCustomerAccountAdd( $payload )
    {
        $app = $this->application;

        $customer = ArrayUtils::get($payload, 'newCustomer');
        if (Validate::isLoadedObject($customer)) {
            $receivers = $app->getNotificationEmails();

            try {
                foreach( $receivers as $receiver ) {
                    Mail::Send(
                        self::TEMPLATE_CUSTOMER_SIGNUP,
                        'New customer signup at ' . Configuration::get('SITE_NAME'),
                        array(
                            'profile_url' => Configuration::get('SITE_DOMAIN', PROX_ADMIN_SITE_ID) . '/customers/edit/' . $customer->id
                        ),
                        $receiver,
                        null,
                        null,
                        null,
                        null,
                        null,
                        CURRENT_CSE_MODULE_DIR . '/mails/',
                        null,
                        null,
                        null,
                        null,
                        PROX_ADMIN_SITE_ID
                    );
                }
            } catch(\Exception $e) {
            }
        }

        return true;
    }
}
