<?php
use Proxim\Application;
use Proxim\Configuration;
use Proxim\Module\Module;
use Proxim\Site\Site;
use Proxim\Util\ArrayUtils;
use Proxim\Validate;

class Google_Tag_Manager extends Module
{
    const GTM_ANALYTICS_ID = 'GTM_ANALYTICS_ID';
    const GTM_ID = 'GTM_ID';

    public function __construct()
    {
        $this->name = 'google_tag_manager';
        $this->icon = 'fab fa-google';
        $this->version = '1.0.0';
        $this->prox_versions_compliancy = array('min' => '1.0.0', 'max' => Application::VERSION);
        $this->author = 'Davison Pro';

        $this->bootstrap = true;
        parent::__construct();

        $this->displayName = 'Google Tag Manager';
        $this->description = 'Manage and deploy analytics and marketing tags as well as other code snippets.';
    }

    /**
     * Echoes a template.
     *
     * @param string $templateName Template name
     */
    public function showTemplate($templateName)
    {
        echo $this->getTemplateContent($templateName);
    }

    /**
     * Return a template.
     *
     * @param string $templateName          Template name
     * @param array  $additionnalParameters Additionnal parameters to inject on the Twig template
     *
     * @return string Parsed template
     */
    private function getTemplateContent($templateName, $additionnalParameters = array())
    {
        $this->smarty->assign($additionnalParameters);
        return $this->fetch(__DIR__ . '/views/' . $templateName.'.tpl');
    }

    public function hookDisplayClientHeader() {
        $application = $this->application;

        $tagmanagerConfiguration = Configuration::getMultiple([
                self::GTM_ANALYTICS_ID,
                self::GTM_ID
            ],
            $application->container->site->id
        );

        $this->smarty->assign([
            'tagmanagerConfiguration' => $tagmanagerConfiguration
        ]);
        
        return $this->getTemplateContent('head');
    }
}
