<?php
use Proxim\Application;
use Proxim\Configuration;
use Proxim\Module\Module;
use Proxim\Site\Site;
use Proxim\Util\ArrayUtils;
use Proxim\Validate;
use Sendpulse\RestApi\ApiClient;
use Sendpulse\RestApi\Storage\FileStorage;

require_once dirname(__FILE__) . '/vendor/autoload.php';

class SendPulse extends Module
{
    const SENDPULSE_ENABLED = 'SENDPULSE_ENABLED';
    const SENDPULSE_API_USER_ID = 'SENDPULSE_API_USER_ID';
    const SENDPULSE_API_SECRET = 'SENDPULSE_API_SECRET';

    public function __construct()
    {
        $this->name = 'sendpulse';
        $this->icon = 'fa fa-envelope';
        $this->version = '1.0.0';
        $this->prox_versions_compliancy = array('min' => '1.0.0', 'max' => Application::VERSION);
        $this->author = 'Davison Pro';

        $this->bootstrap = true;
        parent::__construct();

        $this->displayName = 'SendPulse';
        $this->description = 'Send Email Newsletters, Password Resets, Promotional Emails, Shipping Notifications With Confidence';
    }

    public function install()
    {
        if (!parent::install()) {
            return false;
        }

        $this->registerHook([
            'actionAfterSiteSettingsGeneralUpdate',
            'actionEmailSendBefore',
            'actionMailAlterMessageBeforeSend'
        ]);
    }

    public function hookActionMailAlterMessageBeforeSend($payload) {
        if(!Configuration::get(self::SENDPULSE_ENABLED)) {
            return true;
        }

        $message = ArrayUtils::get($payload, 'message');
        $fileAttachment = ArrayUtils::get($payload, 'fileAttachment');
        $htmlMessage = ArrayUtils::get($payload, 'htmlMessage');

        $from = $message->getFrom();
        $to = $message->getTo();

        $fromAddress = $toAddress = $attachment = array();
        if (is_array($from)) {
            foreach ($from as $addr => $addrName) {
                $addr = trim($addr);
                $fromAddress = array(
                    'name' => $addrName,
                    'email' => $addr
                );
            }
        }

        if (is_array($to)) {
            foreach ($to as $addr => $addrName) {
                $addr = trim($addr);
                $toAddress[] = array(
                    'name' => $addrName,
                    'email' => $addr
                );
            }
        }

        if ($fileAttachment && !empty($fileAttachment)) {
            // Multiple attachments?
            if (!is_array(current($fileAttachment))) {
                $fileAttachment = array($fileAttachment);
            }

            foreach ($fileAttachment as $att) {
                $attachment[$att['name']] = file_get_contents($att['file_path']);
            }
        }

        $email = array(
            'html' => $htmlMessage,
            'subject' => $message->getSubject(),
            'from' => $fromAddress,
            'to' => $toAddress,
            'attachments' => $attachment
        );

        try {
            $sendpulseClient = new ApiClient(
                Configuration::get(self::SENDPULSE_API_USER_ID, PROX_ADMIN_SITE_ID), 
                Configuration::get(self::SENDPULSE_API_SECRET, PROX_ADMIN_SITE_ID), 
                new FileStorage()
            );
            $sendpulseClient->smtpSendMail($email);
        } catch (Exception $e) {
        }

        return false;
    }
}
