<?php
use Proxim\Application;
use Proxim\Configuration;
use Proxim\Module\Module;
use Proxim\Site\Site;
use Proxim\Util\ArrayUtils;
use Proxim\Validate;

class Smartsupp extends Module
{
    const SMARTSUPP_ENABLED = 'SMARTSUPP_ENABLED';
    const PROX_SMARTSUPP_CODE = 'PROX_SMARTSUPP_CODE';

    public function __construct()
    {
        $this->name = 'smartsupp';
        $this->icon = 'fa fa-envelope';
        $this->version = '1.0.0';
        $this->prox_versions_compliancy = array('min' => '1.0.0', 'max' => Application::VERSION);
        $this->author = 'Davison Pro';
        $this->displayName = 'Smartsupp';
        $this->description = 'Smartsupp combines live chat, chatbots, and video recordings to help save you time and effectively convert visitors into loyal customers.';

        $this->bootstrap = true;
        parent::__construct();
    }
    /**
     * Echoes a template.
     *
     * @param string $templateName Template name
     */
    public function showTemplate($templateName)
    {
        echo $this->getTemplateContent($templateName);
    }

    /**
     * Return a template.
     *
     * @param string $templateName          Template name
     * @param array  $additionnalParameters Additionnal parameters to inject on the Twig template
     *
     * @return string Parsed template
     */
    private function getTemplateContent($templateName, $additionnalParameters = array())
    {
        $this->smarty->assign($additionnalParameters);
        return $this->fetch(__DIR__ . '/views/' . $templateName.'.tpl');
    }

    public function hookDisplayClientHeader() {
        $application = $this->application;

        $smartsuppConfiguration = Configuration::getMultiple([
                self::SMARTSUPP_ENABLED,
                self::PROX_SMARTSUPP_CODE
            ],
            $application->container->site->id
        );

        $this->smarty->assign([
            'smartsuppConfiguration' => $smartsuppConfiguration
        ]);
        
        return $this->getTemplateContent('head');
    }
}
